# Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
#
# See file LICENSE for full license details.

import logging
import wirepas_mesh_messaging as wmm

from gi.repository import GLib
from .return_code import ReturnCode


DBUS_SINK_PREFIX = "com.wirepas.sink."


class Sink:
    def __init__(self, bus, proxy, sink_id, unique_name, on_stack_started, on_stack_stopped):

        self.proxy = proxy
        self.sink_id = sink_id
        self.network_address = None
        self.on_stack_started = on_stack_started
        self.on_stack_stopped = on_stack_stopped
        self.bus = bus
        self.unique_name = unique_name
        self._on_started_handle = None
        self._on_stopped_handle = None
        self._last_config_dict = None

    def register_for_stack_started(self):
        # Use the subscribe directly to be able to specify the sender
        self._on_started_handle = self.bus.subscribe(
            signal="StackStarted",
            object="/com/wirepas/sink",
            iface="com.wirepas.sink.config1",
            sender=self.unique_name,
            signal_fired=self._on_stack_started,
        )

    def unregister_from_stack_started(self):
        if self._on_started_handle is not None:
            self._on_started_handle.unsubscribe()

    def register_for_stack_stopped(self):
        # Use the subscribe directly to be able to specify the sender
        self._on_stopped_handle = self.bus.subscribe(
            signal="StackStopped",
            object="/com/wirepas/sink",
            iface="com.wirepas.sink.config1",
            sender=self.unique_name,
            signal_fired=self._on_stack_stopped,
        )

    def unregister_from_stack_stopped(self):
        if self._on_stopped_handle is not None:
            self._on_stopped_handle.unsubscribe()

    def get_network_address(self, force=False):
        if self.network_address is None or force:
            # Network address is not known or must be updated
            try:
                self.network_address = self.proxy.NetworkAddress
            except GLib.Error:
                logging.error("Could not get network address")

        return self.network_address

    def send_data(
        self,
        dst,
        src_ep,
        dst_ep,
        qos,
        initial_time,
        data,
        is_unack_csma_ca=False,
        hop_limit=0,
    ):
        try:
            res = self.proxy.SendMessage(
                # For some reason on some arch, uint32 are not correctly handled
                dst & 0xFFFFFFFF,
                src_ep,
                dst_ep,
                initial_time,
                qos,
                is_unack_csma_ca,
                hop_limit,
                data,
            )
            if res != 0:
                logging.error("Cannot send message err=%s", res)
                return ReturnCode.error_from_dbus_return_code(res)
        except GLib.Error as e:
            logging.error("Fail to send message: %s", str(e))
            return ReturnCode.error_from_dbus_exception(str(e))
        except OverflowError:
            # It may happens as protobuf has bigger container value
            logging.error("Invalid range value")
            return wmm.GatewayResultCode.GW_RES_INVALID_PARAM

        return wmm.GatewayResultCode.GW_RES_OK

    def _on_stack_started(self, sender, object, iface, signal, params):
        # pylint: disable=unused-argument
        # pylint: disable=redefined-builtin
        # Force update of network address in case remote api modify it
        self.get_network_address(True)

        self.on_stack_started(self.sink_id)

    def _on_stack_stopped(self, sender, object, iface, signal, params):
        self.on_stack_stopped(self.sink_id)

    def _get_param(self, dic, key, attribute):
        try:
            dic[key] = getattr(self.proxy, attribute)
        except  GLib.Error:
            if key != "channel_map":
                # Warning and not an error as normal behavior if not set
                logging.warning("Cannot get %s in config (is it set?)", key)

        except AttributeError :
            logging.warning("Attribute %s doesn't exist", key)

    def _get_pair_params(self, dic, key1, att1, key2, att2):
        # Some settings are only relevant if the both can be retrieved
        try:
            att1_val = getattr(self.proxy, att1)
            att2_val = getattr(self.proxy, att2)
        except GLib.Error:
            logging.debug("Cannot get one of the pair value (%s-%s)", key1, key2)
            return

        dic[key1] = att1_val
        dic[key2] = att2_val

    def read_config(self):
        config = {}
        config["sink_id"] = self.sink_id
        partial = False

        # Should always be available
        try:
            config["started"] = (self.proxy.StackStatus & 0x01) == 0
        except GLib.Error as e:
            error = ReturnCode.error_from_dbus_exception(str(e))
            logging.error("Cannot get Stack state: %s", error)

        self._get_param(config, "node_address", "NodeAddress")
        self._get_param(config, "node_role", "NodeRole")
        self._get_param(config, "network_address", "NetworkAddress")
        self._get_param(config, "network_channel", "NetworkChannel")
        self._get_param(config, "channel_map", "ChannelMap")
        self._get_pair_params(config, "max_ac", "ACRangeMax", "min_ac", "ACRangeMin")
        self._get_pair_params(
            config, "max_ac_cur", "ACRangeMaxCur", "min_ac_cur", "ACRangeMinCur"
        )
        self._get_pair_params(config, "max_ch", "ChRangeMax", "min_ch", "ChRangeMin")
        self._get_param(config, "max_mtu", "MaxMtu")
        self._get_param(config, "hw_magic", "HwMagic")
        self._get_param(config, "stack_profile", "StackProfile")
        self._get_param(config, "firmware_version", "FirmwareVersion")
        self._get_param(config, "app_config_max_size", "AppConfigMaxSize")

        try:
            are_keys_set = self.proxy.AuthenticationKeySet and self.proxy.CipherKeySet

            config["are_keys_set"] = are_keys_set
        except GLib.Error:
            logging.error("Cannot get key status")

        try:
            seq, diag, data = self.proxy.GetAppConfig()
            config["app_config_seq"] = seq
            config["app_config_diag"] = diag
            config["app_config_data"] = bytearray(data)
        except GLib.Error:
            # If node is blank it is not a sink
            # so app config cannot be accessed
            logging.warning("Cannot get App Config")

        # Add scratchpad related info
        self.get_scratchpad_status(config)

        if self._last_config_dict is not None:
            for key, value in self._last_config_dict.items():
                if key not in config:
                    logging.warning("Add %s from cache in config", key)
                    config[key] = value
                    partial = True

        self._last_config_dict = config.copy()

        return config, partial

    def _set_param(self, dic, key, attribute):
        try:
            value = dic[key]
            # Stop the stack if not already stopped
            if self.proxy.StackStatus == 0:
                self.proxy.SetStackState(False)

            setattr(self.proxy, attribute, value)

        except KeyError:
            # key not defined in config
            logging.debug("key not present: %s", key)
        except GLib.Error as e:
            # Exception raised when setting attribute
            logging.error(
                "Cannot set %s for param %s on sink %s: %s",
                value,
                key,
                self.sink_id,
                str(e),
            )
            return ReturnCode.error_from_dbus_exception(str(e))
        except OverflowError:
            # It may happens as protobuf has bigger container value
            logging.error(
                "Invalid range value for param %s with value %s", key, value
            )
            return wmm.GatewayResultCode.GW_RES_INVALID_PARAM

        return wmm.GatewayResultCode.GW_RES_OK

    def write_config(self, config):
        # Force the node address if used
        try:
            # For some reason on some arch, uint32 are not correctly handled and result to an overflow
            config["node_address"] = config["node_address"] & 0xFFFFFFFF
        except KeyError:
            # Node addess is not in the config
            pass
        # Should always be available
        try:
            stack_started = (self.proxy.StackStatus & 0x01) == 0
        except GLib.Error as e:
            res = ReturnCode.error_from_dbus_exception(str(e))
            logging.error(
                "Cannot get Stack state. Problem in communication probably: %s",
                res.name,
            )
            return res

        # The write config has only one return code possible
        # so the last error code will be returned
        res = wmm.GatewayResultCode.GW_RES_OK

        config_to_dbus_param = dict(
            [
                ("node_address", "NodeAddress"),
                ("node_role", "NodeRole"),
                ("network_address", "NetworkAddress"),
                ("network_channel", "NetworkChannel"),
                ("channel_map", "ChannelMap"),
                ("authentication_key", "AuthenticationKey"),
                ("cipher_key", "CipherKey"),
            ]
        )

        # Any following call will stop the stack
        for param in config_to_dbus_param:
            tmp = self._set_param(config, param, config_to_dbus_param[param])
            if tmp != wmm.GatewayResultCode.GW_RES_OK:
                # Update result code only if not success to avoid erasing
                # previous error (only one return code)
                res = tmp

        # Set app_config after node role config in case role was not sink before
        try:
            seq = config["app_config_seq"]
            diag = config["app_config_diag"]
            data = config["app_config_data"]

            logging.info("Set app config with %s", config)
            self.proxy.SetAppConfig(seq, diag, data)
        except KeyError:
            # App config not defined in new config
            logging.debug("Missing key app_config key in config: %s", config)
        except GLib.Error as e:
            res = ReturnCode.error_from_dbus_exception(str(e))
            logging.error("Cannot set App Config: %s", res.name)
        except OverflowError:
            # It may happens as protobuf has bigger container value
            res = wmm.GatewayResultCode.GW_RES_INVALID_PARAM
            logging.error("Invalid range value")

        # Set stack in state defined by new config or set it as it was
        # previously
        try:
            new_state = config["started"]
        except KeyError:
            # Not defined in config
            new_state = stack_started

        try:
            current_state = (self.proxy.StackStatus & 0x01) == 0
            # Change stack state only if needed to avoid unnecessary events
            if current_state != new_state:
                logging.debug(
                    "Change stack state from %s to %s", current_state, new_state
                )
                self.proxy.SetStackState(new_state)
        except GLib.Error as err:
            res = ReturnCode.error_from_dbus_exception(str(err))
            logging.error(
                "Cannot set Stack state. Problem in communication probably: %s",
                res.name,
            )
            return res

        # In case the network address was updated, read it back for our cached
        # value
        self.get_network_address(True)

        return res

    @property
    def cost(self):
        try:
            cost = self.proxy.SinkCost
        except GLib.Error:
            logging.error("Cannot get sink cost for sink {}".format(self.sink_id))
            cost = 0
        return cost

    @cost.setter
    def cost(self, new_cost):
        if new_cost is None or new_cost < 0 or new_cost > 254:
            raise ValueError("Wrong sink cost value {}".format(new_cost))

        try:
            self.proxy.SinkCost = new_cost
        except GLib.Error as err:
            res = ReturnCode.error_from_dbus_exception(err.message)
            if res == wmm.GatewayResultCode.GW_RES_INVALID_ROLE:
                logging.warning("Node role is not a sink, sink cost cannot be modified")
                raise ValueError("Wrong role to set cost value {}".format(new_cost))
            else:
                logging.error("Cannot set sink cost for sink {} ({})".format(self.sink_id, res))

    def get_scratchpad_status(self, out_d=None):
        if out_d is None:
            d = {}
        else:
            d = out_d

        dbus_to_gateway_satus = dict(
            [
                (0, wmm.ScratchpadStatus.SCRATCHPAD_STATUS_SUCCESS),
                (255, wmm.ScratchpadStatus.SCRATCHPAD_STATUS_NEW)
                # Anything else is ERROR
            ]
        )
        try:
            status = self.proxy.StoredStatus
            d["stored_status"] = dbus_to_gateway_satus[status]
        except GLib.Error:
            # Exception raised when getting attribute (probably not set)
            logging.error("Cannot get stored status in config")
        except KeyError:
            # Between 1 and 254 => Error
            logging.error("Scratchpad stored status has error: %s", status)
            d["stored_status"] = wmm.ScratchpadStatus.SCRATCHPAD_STATUS_ERROR

        dbus_to_gateway_type = dict(
            [
                (0, wmm.ScratchpadType.SCRATCHPAD_TYPE_BLANK),
                (1, wmm.ScratchpadType.SCRATCHPAD_TYPE_PRESENT),
                (2, wmm.ScratchpadType.SCRATCHPAD_TYPE_PROCESS),
            ]
        )
        try:
            stored_type = self.proxy.StoredType
            d["stored_type"] = dbus_to_gateway_type[stored_type]
        except GLib.Error:
            # Exception raised when getting attribute (probably not set)
            logging.error("Cannot get stored type in config\n")

        stored = {}
        self._get_param(stored, "seq", "StoredSeq")
        self._get_param(stored, "crc", "StoredCrc")
        self._get_param(stored, "len", "StoredLen")
        d["stored_scratchpad"] = stored

        processed = {}
        self._get_param(processed, "seq", "ProcessedSeq")
        self._get_param(processed, "crc", "ProcessedCrc")
        self._get_param(processed, "len", "ProcessedLen")
        d["processed_scratchpad"] = processed

        self._get_param(d, "firmware_area_id", "FirmwareAreaId")

        # Read Target only if firmware is greater than 5.1
        stack_version = self.proxy.FirmwareVersion
        if stack_version[0] > 5 or (stack_version[0] == 5 and stack_version[1] > 0):
            # Target scratchpad should be supported (except if dualmcu is too old)
            try:
                seq, crc, action, param = self.proxy.GetTargetScratchpad()
                target_and_action = {}
                target_and_action["action"] = wmm.ScratchpadAction(
                    action + 1
                )  # Plus one as dualmcu version starts at 0, and we start at 1
                target_and_action["target_sequence"] = seq
                target_and_action["target_crc"] = crc
                if target_and_action["action"] == wmm.ScratchpadAction.ACTION_PROPAGATE_AND_PROCESS_WITH_DELAY:
                    # check if delay is in predifined list
                    param_to_delay_dic = {
                        0x4A: wmm.ProcessingDelay.DELAY_TEN_MINUTES,
                        0x5E: wmm.ProcessingDelay.DELAY_THIRTY_MINUTES,
                        0x81: wmm.ProcessingDelay.DELAY_ONE_HOUR,
                        0x86: wmm.ProcessingDelay.DELAY_SIX_HOURS,
                        0xC1: wmm.ProcessingDelay.DELAY_ONE_DAY,
                        0xC2: wmm.ProcessingDelay.DELAY_TWO_DAYS,
                        0xC5: wmm.ProcessingDelay.DELAY_FIVE_DAYS
                    }

                    try:
                        target_and_action["delay"] = param_to_delay_dic[param]
                    except KeyError:
                        logging.debug("Delay not in predifined list: %d", param)
                        target_and_action["param"] = param

                else:
                    target_and_action["param"] = param

                d["target_and_action"] = target_and_action
            except GLib.Error:
                logging.warning("Cannot get Target Scratchpad")

        return d

    def process_scratchpad(self):
        ret = wmm.GatewayResultCode.GW_RES_OK
        restart = False
        try:
            # Stop the stack if not already stopped
            if self.proxy.StackStatus == 0:
                self.proxy.SetStackState(False)
                restart = True
        except GLib.Error:
            logging.error("Sink in invalid state")
            return wmm.GatewayResultCode.GW_RES_INVALID_SINK_STATE

        try:
            self.proxy.ProcessScratchpad()
        except GLib.Error as e:
            ret = ReturnCode.error_from_dbus_exception(str(e))
            logging.error("Could not restore sink's state: %s", ret.name)

        if restart:
            try:
                self.proxy.SetStackState(True)
            except GLib.Error as e:
                ret = ReturnCode.error_from_dbus_exception(str(e))
                logging.debug("Sink in invalid state: %s", ret.name)

        return ret

    def upload_scratchpad(self, seq, file):
        ret = wmm.GatewayResultCode.GW_RES_OK
        restart = False
        try:
            # Stop the stack if not already stopped
            if self.proxy.StackStatus == 0:
                self.proxy.SetStackState(False)
                restart = True
        except GLib.Error:
            logging.error("Sink in invalid state")
            return wmm.GatewayResultCode.GW_RES_INVALID_SINK_STATE

        try:
            self.proxy.UploadScratchpad(seq, file)
            logging.info(
                "Scratchpad loaded with seq %d on sink %s", seq, self.sink_id
            )
        except GLib.Error as e:
            ret = ReturnCode.error_from_dbus_exception(str(e))
            logging.error("Cannot upload local scratchpad: %s", ret.name)
        except OverflowError:
            # It may happens as protobuf has bigger container value
            ret = wmm.GatewayResultCode.GW_RES_INVALID_PARAM
            logging.error("Invalid range value")

        if restart:
            try:
                # Restart sink if we stopped it for this request
                self.proxy.SetStackState(True)
            except GLib.Error as e:
                ret = ReturnCode.error_from_dbus_exception(str(e))
                logging.error("Could not restore sink's state: %s", ret.name)

        return ret

    def set_target_scratchpad(self, action, target_seq, target_crc, param):
        ret = wmm.GatewayResultCode.GW_RES_OK

        if (
            action == wmm.ScratchpadAction.ACTION_NO_OTAP
            or action == wmm.ScratchpadAction.ACTION_LEGACY_OTAP
        ):
            # There is no target with those actions, default to 0
            target_seq = 0
            target_crc = 0
            param = 0
        else:
            # check params in case there is an action with valid target
            try:
                if target_seq is None:
                    # Target seq not specified, take local one
                    target_seq = self.proxy.StoredSeq

                if target_crc is None:
                    # Target crc not specified, take local one
                    target_crc = self.proxy.StoredCrc
            except GLib.Error as e:
                ret = ReturnCode.error_from_dbus_exception(str(e))
                logging.error(
                    "Cannot get local scratchpad info for set_target: %s", ret.name
                )

            if target_seq == 0:
                logging.error("Seq 0 is not a valid target")
                return wmm.GatewayResultCode.GW_RES_INVALID_PARAM

        try:
            # If there is no param for the action, default it to 0 before
            # calling sink service through DBUS
            if param is None:
                param = 0

            logging.info(
                "Scratchpad target set to Action %s (%d) with seq"
                " %d and crc %d on sink %s",
                action,
                param,
                target_seq,
                target_crc,
                self.sink_id,
            )

            # Do a minus 1 as action are shifted by one with dual mcu
            self.proxy.SetTargetScratchpad(
                target_seq, target_crc, action.value - 1, param
            )

        except GLib.Error as e:
            ret = ReturnCode.error_from_dbus_exception(str(e))
            logging.warning("Cannot set target scratchpad: %s (can be normal if same value as already set)", ret.name)

        return ret


class SinkManager:
    "Helper class to manage the Sink list"

    def __init__(
        self, bus, on_new_sink_cb, on_sink_removal_cb, on_stack_started, on_stack_stopped
    ):

        self.sinks = {}
        # List used to quickly retrieved sink well known name
        self.sender_to_name = {}
        self.bus = bus

        self.add_cb = None
        self.rm_cb = None
        self.stack_started_cb = on_stack_started
        self.stack_stopped_cb = on_stack_stopped

        bus_monitor = self.bus.get("org.freedesktop.DBus")

        # Find sinks already on bus
        for name in bus_monitor.ListNames():
            if name.startswith(DBUS_SINK_PREFIX):
                short_name = name[len(DBUS_SINK_PREFIX) :]
                self._add_sink(short_name, bus_monitor.GetNameOwner(name))

        # Monitor the bus for connections
        self.bus.subscribe(
            sender="org.freedesktop.DBus",
            signal="NameOwnerChanged",
            signal_fired=self._on_name_owner_changed,
        )

        # Set them at the end to be sure Sink Manager is ready when cb are fired
        self.add_cb = on_new_sink_cb
        self.rm_cb = on_sink_removal_cb

    def _add_sink(self, short_name, unique_name):
        if short_name in self.sinks:
            logging.warning("Sink already in list sink name=%s", short_name)
            return

        # Open proxy for this sink
        proxy = self.bus.get(
            DBUS_SINK_PREFIX + short_name,  # Bus name
            "/com/wirepas/sink",  # Object path
        )

        sink = Sink(
            bus=self.bus,
            proxy=proxy,
            sink_id=short_name,
            unique_name=unique_name,
            on_stack_started=self.stack_started_cb,
            on_stack_stopped=self.stack_stopped_cb,
        )

        sink.register_for_stack_started()
        sink.register_for_stack_stopped()

        self.sinks[short_name] = sink

        self.sender_to_name[unique_name] = short_name

        if self.add_cb is not None:
            self.add_cb(short_name)

        logging.info("New sink added with name %s", short_name)

    def _remove_sink(self, short_name):
        try:
            sink = self.sinks.pop(short_name)
            sink.unregister_from_stack_started()
            sink.unregister_from_stack_stopped()

            # Remove Sink to association list
            for k, v in self.sender_to_name.items():
                if v == short_name:
                    self.sender_to_name.pop(k)
                    logging.warning("Association removed from %s => %s", k, v)
                    break

            # call client cb
            if self.rm_cb is not None:
                self.rm_cb(short_name)
        except KeyError:
            logging.error("Cannot remove %s from sink list", short_name)

        logging.info("Sink removed with name %s", short_name)

    def _on_name_owner_changed(self, sender, object, iface, signal, params):
        # pylint: disable=unused-argument
        # pylint: disable=redefined-builtin
        well_known_name = params[0]
        if well_known_name.startswith(DBUS_SINK_PREFIX):
            short_name = well_known_name[len(DBUS_SINK_PREFIX) :]
            # Owner change on a sink, check if it is removal or addition
            old_owner = params[1]
            new_owner = params[2]
            if old_owner == "":
                # New sink connection
                self._add_sink(short_name, new_owner)
            elif new_owner == "":
                # Removal
                self._remove_sink(short_name)
            else:
                logging.critical(
                    "Not addition nor removal ??? %s: %s => %s",
                    well_known_name,
                    old_owner,
                    new_owner,
                )

    def get_sinks(self):
        # Return a list that is a copy to avoid modification
        # of list while iterating on it (if new sink is connected)
        return list(self.sinks.values())

    def get_sink_name(self, bus_name):
        try:
            return self.sender_to_name[bus_name]
        except KeyError:
            logging.error("Unknown sink %s from sink list", bus_name)
            return None

    def get_sink(self, short_name):
        try:
            return self.sinks[short_name]
        except KeyError:
            logging.error("Unknown sink %s from sink list", short_name)
            return None



"""
    Wirepas Gateway
    ===============

    .. Style guidelines:
       http://google.github.io/styleguide/pyguide.html

    .. Copyright:
        Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
        See file LICENSE for full license details.
"""
# flake8: noqa

from . import dbus
from . import protocol
from . import utils

from .__about__ import (
    __author__,
    __author_email__,
    __classifiers__,
    __copyright__,
    __description__,
    __license__,
    __pkg_name__,
    __title__,
    __url__,
    __version__,
    __keywords__,
)



"""
    Serialization tools
    ===================

    Contains multipurpose utilities for serializing objects.

    .. Copyright:
        Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
        See file LICENSE for full license details.
"""

import json
import datetime
import binascii


def serialize(obj) -> str:
    """ Serializes an object into json """
    return json.dumps(obj, default=json_serial, sort_keys=True, indent=4)


def json_serial(obj) -> str:
    """JSON serializer for objects not serializable by default json code"""

    if isinstance(obj, (datetime.datetime, datetime.date)):
        return obj.isoformat()

    if isinstance(obj, (bytearray, bytes)):
        return binascii.hexlify(obj)
    if isinstance(obj, set):
        return str(obj)

    raise TypeError("Type %s not serializable" % type(obj))



"""
    __MAIN__
    ========

    Package main file

    .. Copyright:
        Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
        See file LICENSE for full license details.
"""

from wirepas_gateway.transport_service import main as main_transport


def main():
    main_transport()


if __name__ == "__main__":
    main()



# Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
#
# See file LICENSE for full license details.
#
BASE_GW_EVENT = "gw-event"
BASE_REQUEST = "gw-request"
BASE_RESPONSE = "gw-response"


class TopicGenerator:
    """
        Static class used as a helper to isolate the mqtt topic part

        It generates string topic from parameters
    """

    @staticmethod
    def _make_topic(base, cmd, params):
        req = base + "/" + cmd

        for param in params:
            req += "/" + param

        return req

    ##################
    # Requests Part
    ##################
    @staticmethod
    def _make_request_topic(cmd, params):
        return TopicGenerator._make_topic(BASE_REQUEST, cmd, params)

    @staticmethod
    def make_get_configs_request_topic(gw_id="+"):
        return TopicGenerator._make_request_topic("get_configs", [str(gw_id)])

    @staticmethod
    def make_set_config_request_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_request_topic(
            "set_config", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_send_data_request_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_request_topic(
            "send_data", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_otap_status_request_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_request_topic(
            "otap_status", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_otap_load_scratchpad_request_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_request_topic(
            "otap_load_scratchpad", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_otap_process_scratchpad_request_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_request_topic(
            "otap_process_scratchpad", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_otap_set_target_scratchpad_request_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_request_topic(
            "otap_set_target_scratchpad", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_get_gateway_info_request_topic(gw_id):
        return TopicGenerator._make_request_topic("get_gw_info", [str(gw_id)])

    ##################
    # Response Part
    ##################
    @staticmethod
    def _make_response_topic(cmd, params):
        return TopicGenerator._make_topic(BASE_RESPONSE, cmd, params)

    @staticmethod
    def make_get_configs_response_topic(gw_id="+"):
        return TopicGenerator._make_response_topic("get_configs", [str(gw_id)])

    @staticmethod
    def make_set_config_response_topic(gw_id, sink_id):
        return TopicGenerator._make_response_topic(
            "set_config", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_send_data_response_topic(gw_id, sink_id):
        return TopicGenerator._make_response_topic(
            "send_data", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_otap_status_response_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_response_topic(
            "otap_status", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_otap_upload_scratchpad_response_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_response_topic(
            "otap_load_scratchpad", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_otap_process_scratchpad_response_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_response_topic(
            "otap_process_scratchpad", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_otap_set_target_scratchpad_response_topic(gw_id="+", sink_id="+"):
        return TopicGenerator._make_response_topic(
            "otap_set_target_scratchpad", [str(gw_id), str(sink_id)]
        )

    @staticmethod
    def make_get_gateway_info_response_topic(gw_id):
        return TopicGenerator._make_response_topic("get_gw_info", [str(gw_id)])

    ##################
    # Event Part
    ##################
    @staticmethod
    def _make_event_topic(cmd, params):
        return TopicGenerator._make_topic(BASE_GW_EVENT, cmd, params)

    @staticmethod
    def make_status_topic(gw_id="+"):
        return TopicGenerator._make_event_topic("status", [str(gw_id)])

    @staticmethod
    def make_received_data_topic(
        gw_id="+", sink_id="+", network_id="+", src_ep="+", dst_ep="+"
    ):
        return TopicGenerator._make_event_topic(
            "received_data",
            [str(gw_id), str(sink_id), str(network_id), str(src_ep), str(dst_ep)],
        )


class TopicParser:
    """
        Static class used as a helper to parse topic

        It parses parameters from topic string topic
        """

    @staticmethod
    def parse_send_data_topic(topic):
        _, cmd, gw_id, sink_id = topic.split("/")
        if not cmd.startswith("send_data"):
            raise RuntimeError("Wrong topic for send_data_request")

        return gw_id, sink_id



# Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
#
# See file LICENSE for full license details.

import logging
import queue
import socket
import ssl
from select import select
from threading import Thread, Lock
from time import sleep
from datetime import datetime

from paho.mqtt import client as mqtt
from paho.mqtt.client import connack_string


class MQTTWrapper(Thread):
    """
    Class to manage the MQTT main thread and be able to share it with other services
    In this case, it allows to have all the related mqtt activity happening on same thread
    to avoid any dead lock from mqtt client.
    """

    # Keep alive time with broker
    KEEP_ALIVE_S = 20

    def __init__(
        self,
        settings,
        on_termination_cb=None,
        on_connect_cb=None,
        last_will_topic=None,
        last_will_data=None,
    ):
        Thread.__init__(self)
        self.daemon = True
        self.running = False
        self.on_termination_cb = on_termination_cb
        self.on_connect_cb = on_connect_cb
        # Set to track the unpublished packets
        self._unpublished_mid_set = set()
        # Keep track of latest published packet
        self._publish_monitor = PublishMonitor()

        if settings.mqtt_use_websocket:
            transport = "websockets"
            self._use_websockets = True
        else:
            transport = "tcp"
            self._use_websockets = False

        self._client = mqtt.Client(
            client_id=settings.gateway_id,
            clean_session=not settings.mqtt_persist_session,
            transport=transport,
        )

        if not settings.mqtt_force_unsecure:
            try:
                self._client.tls_set(
                    ca_certs=settings.mqtt_ca_certs,
                    certfile=settings.mqtt_certfile,
                    keyfile=settings.mqtt_keyfile,
                    cert_reqs=ssl.VerifyMode[settings.mqtt_cert_reqs],
                    tls_version=ssl._SSLMethod[settings.mqtt_tls_version],
                    ciphers=settings.mqtt_ciphers,
                )
            except Exception as e:
                logging.error("Cannot use secure authentication %s", e)
                exit(-1)

        logging.info(
            "Max inflight messages set to %s", settings.mqtt_max_inflight_messages
        )
        self._client.max_inflight_messages_set(settings.mqtt_max_inflight_messages)

        self._client.username_pw_set(settings.mqtt_username, settings.mqtt_password)
        self._client.on_connect = self._on_connect
        self._client.on_publish = self._on_publish
        self._client.on_disconnect = self._on_disconnect

        if last_will_topic is not None and last_will_data is not None:
            self._set_last_will(last_will_topic, last_will_data)

        try:
            self._client.connect(
                settings.mqtt_hostname,
                settings.mqtt_port,
                keepalive=MQTTWrapper.KEEP_ALIVE_S,
            )
        except (socket.gaierror, ValueError) as e:
            logging.error(
                "Error on MQTT address %s:%d => %s"
                % (settings.mqtt_hostname, settings.mqtt_port, str(e))
            )
            # Do not exit as it and let the retry mechanism
            # to reconnect. It can happen if connection is not available yet.
            # mqtt_reconnect_delay setting can be used to limit the retry.
        except ConnectionRefusedError:
            logging.error("Connection Refused by MQTT broker")
            exit(-1)
        except OSError as e:
            logging.error("Cannot establish connection (%s)", str(e))
            # It will happen if broker is down when trying first connection.
            # But it can happen also if settings are wrong (host or port)
            # mqtt_reconnect_delay setting can be used to limit the retry.

        self.timeout = settings.mqtt_reconnect_delay

        # Set options to initial socket if tcp transport only
        if not self._use_websockets and self._client.socket() is not None:
            self._client.socket().setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2048)

        self._publish_queue = SelectableQueue()

        # Thread is not started yes
        self.running = False
        self.connected = False

    def _on_connect(self, client, userdata, flags, rc):
        # pylint: disable=unused-argument
        if rc != 0:
            logging.error("MQTT cannot connect: %s (%s)", connack_string(rc), rc)
            self.running = False
            return

        self.connected = True
        if self.on_connect_cb is not None:
            self.on_connect_cb()

    def _on_disconnect(self, userdata, rc):
        if rc != 0:
            logging.error(
                "MQTT unexpected disconnection (network or broker originated):"
                "%s (%s)",
                connack_string(rc),
                rc,
            )
            self.connected = False

    def _on_publish(self, client, userdata, mid):
        self._unpublished_mid_set.remove(mid)
        self._publish_monitor.on_publish_done()
        return

    def _do_select(self, sock):
        # Select with a timeout of 1 sec to call loop misc from time to time
        r, w, _ = select(
            [sock, self._publish_queue],
            [sock] if self._client.want_write() else [],
            [],
            1,
        )

        if sock in r:
            self._client.loop_read()

        if sock in w:
            self._client.loop_write()

        self._client.loop_misc()

        # Check if we have something to publish
        if self._publish_queue in r:
            try:
                # Publish everything. Loop is not necessary as
                # next select will exit immediately if queue not empty
                while True:
                    topic, payload, qos, retain = self._publish_queue.get()

                    self._publish_from_wrapper_thread(
                        topic, payload, qos=qos, retain=retain
                    )

                    # FIX: read internal sockpairR as it is written but
                    # never read as we don't use the internal paho loop
                    # but we have spurious timeout / broken pipe from
                    # this socket pair
                    # pylint: disable=protected-access
                    try:
                        self._client._sockpairR.recv(1)
                    except Exception:
                        # This socket is not used at all, so if something is wrong,
                        # not a big issue. Just keep going
                        pass

            except queue.Empty:
                # No more packet to publish
                pass

    def _get_socket(self):
        sock = self._client.socket()
        if sock is not None:
            return sock

        if self.connected:
            logging.error("MQTT Inner loop, unexpected disconnection")

        # Socket is not opened anymore, try to reconnect for timeout if set
        loop_forever = self.timeout == 0
        delay = 0
        logging.info("Starting reconnect loop with timeout %d" % self.timeout)
        # Loop forever or until timeout is over
        while loop_forever or (delay <= self.timeout):
            try:
                logging.debug("MQTT reconnect attempt delay=%d" % delay)
                ret = self._client.reconnect()
                if ret == mqtt.MQTT_ERR_SUCCESS:
                    break
            except Exception:
                # Retry to connect in 1 sec up to timeout if set
                sleep(1)
                delay += 1
                logging.debug("Retrying to connect in 1 sec")

        if not loop_forever:
            # In case of timeout set, check if it exits because of timeout
            if delay > self.timeout:
                logging.error("Unable to reconnect after %s seconds", delay)
                return None

        # Socket must be available once reconnect is successful
        if self._client.socket() is None:
            logging.error("Cannot get socket after reconnect")
            return None
        else:
            logging.info("Successfully acquired socket after reconnect")

        # Set options to new reopened socket
        if not self._use_websockets:
            self._client.socket().setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2048)
        return self._client.socket()

    def _set_last_will(self, topic, data):
        # Set Last wil message
        self._client.will_set(topic, data, qos=2, retain=True)

    def run(self):
        self.running = True

        while self.running:
            try:
                try:
                    # check if we are connected
                    # Get client socket to select on it
                    # This function manage the reconnect
                    sock = self._get_socket()
                    if sock is None:
                        # Cannot get the socket, probably an issue
                        # with connection. Exit the thread
                        logging.error("Cannot get MQTT socket, exit...")
                        self.running = False
                    else:
                        self._do_select(sock)
                except TimeoutError:
                    logging.error("Timeout in connection, force a reconnect")
                    self._client.reconnect()
            except Exception:
                # If an exception is not caught before this point
                # All the transport module must be stopped in order to be fully
                # restarted by the managing agent
                logging.exception("Unexpected exception in MQTT wrapper Thread")
                self.running = False

        if self.on_termination_cb is not None:
            # As this thread is daemonized, inform the parent that this
            # thread has exited
            self.on_termination_cb()

    def _publish_from_wrapper_thread(self, topic, payload, qos, retain):
        """Internal method to publish on Mqtt. This method is only called from
        mqtt wrapper thread to avoid races.

        Args:
            topic: Topic to publish on
            payload: Payload
            qos: Qos to use
            retain: Is it a retain message

        """
        mid = self._client.publish(topic, payload, qos=qos, retain=retain).mid
        self._unpublished_mid_set.add(mid)

    def publish(self, topic, payload, qos=1, retain=False) -> None:
        """ Method to publish to Mqtt from any thread

        Args:
            topic: Topic to publish on
            payload: Payload
            qos: Qos to use
            retain: Is it a retain message

        """
        # Send it to the queue to be published from Mqtt thread
        self._publish_queue.put((topic, payload, qos, retain))
        self._publish_monitor.on_publish_request()

    def subscribe(self, topic, cb, qos=2) -> None:
        logging.debug("Subscribing to: {}".format(topic))
        self._client.subscribe(topic, qos)
        self._client.message_callback_add(topic, cb)

    @property
    def publish_queue_size(self):
        return self._publish_monitor.get_publish_queue_size()

    @property
    def publish_waiting_time_s(self):
        return self._publish_monitor.get_publish_waiting_time_s()


class SelectableQueue(queue.Queue):
    """
    Wrapper arround a Queue to make it selectable with an associated
    socket
    """

    def __init__(self):
        super().__init__()
        self._putsocket, self._getsocket = socket.socketpair()
        self._lock = Lock()
        self._size = 0

    def fileno(self):
        """
        Implement fileno to be selectable
        :return: the reception socket fileno
        """
        return self._getsocket.fileno()

    def put(self, item, block=True, timeout=None):
        with self._lock:
            if self._size == 0:
                # Send 1 byte on socket to signal select
                self._putsocket.send(b"x")
            self._size = self._size + 1

            # Insert item in queue
            super().put(item, block, timeout)

    def get(self, block=False, timeout=None):
        with self._lock:
            # Get item first so get can be called and
            # raise empty exception
            item = super().get(block, timeout)

            self._size = self._size - 1
            if self._size == 0:
                # Consume 1 byte from socket
                self._getsocket.recv(1)
            return item


class PublishMonitor:
    """
        Object dedicated to MQTT publish monitoring, in a simple
        and "Thread-safe" way.
    """

    def __init__(self):
        self._lock = Lock()
        self._size = 0
        self._last_publish_event_timestamp = 0  # valid if size != 0

    def get_publish_queue_size(self):
        with self._lock:
            return self._size

    def get_publish_waiting_time_s(self):
        with self._lock:
            if self._size == 0:
                return 0
            else:
                delta = datetime.now() - self._last_publish_event_timestamp
                return delta.total_seconds()

    def on_publish_request(self):
        with self._lock:
            if self._size == 0:
                self._last_publish_event_timestamp = datetime.now()
            self._size = self._size + 1

    def on_publish_done(self):
        with self._lock:
            self._size = self._size - 1
            self._last_publish_event_timestamp = datetime.now()



#!/usr/bin/env python3
# Copyright 2020 Wirepas Ltd licensed under Apache License, Version 2.0
#
# See file LICENSE for full license details.
#
import binascii
import argparse
import os

from enum import Enum

from wirepas_gateway.dbus.dbus_client import BusClient


class NodeRole:
    """Utility class to package a node role

    Helper to manipulate and convert node role based on a Base role
    and a list of flags

    """

    class BaseRole(Enum):
        SINK = 0x1
        ROUTER = 0x2
        NON_ROUTER = 0x3

    class RoleFlags(Enum):
        CSMA_CA = 0x1
        AUTOROLE = 0x8

    def __init__(self, base, flags):
        """Base role constructor

        Args:
            base: a BaseRole
            flags: list of RoleFlags
        """
        self.base = base
        self.flags = flags

    @classmethod
    def from_string(cls, node_role_str):
        """Create a NodeRole from a string

        Args:
            node_role_str: string containing the node role

        Returns: A NodeRole object
        """
        str_lower = node_role_str.lower()
        if "sink" in str_lower:
            base = cls.BaseRole.SINK
        elif "non-router" in str_lower:
            base = cls.BaseRole.NON_ROUTER
        elif "router" in str_lower:
            base = cls.BaseRole.ROUTER
        else:
            raise ValueError("Cannot determine base role from %s" % node_role_str)

        flags = []
        if "csma-ca" in str_lower:
            flags.append(cls.RoleFlags.CSMA_CA)

        if "autorole" in str_lower:
            flags.append(cls.RoleFlags.AUTOROLE)

        return cls(base, flags)

    @classmethod
    def from_dualmcu_value(cls, val):
        """Create a NodeRole from a dualmcu value

        Args:
            val: node role as a 1 byte value from dual mcu api

        Returns: A NodeRole object
        """
        base_int = val & 0x0F
        flags_int = val & 0xF0

        # Define base role
        if base_int == 1:
            base = cls.BaseRole.SINK
        elif base_int == 2:
            base = cls.BaseRole.ROUTER
        elif base_int == 3:
            base = cls.BaseRole.NON_ROUTER

        # Define flags
        flags = []
        if flags_int & 0x10 != 0:
            flags.append(cls.RoleFlags.CSMA_CA)

        if flags_int & 0x80 != 0:
            flags.append(cls.RoleFlags.AUTOROLE)

        return cls(base, flags)

    def to_dualmcu_value(self):
        """Convert node role to dual mcu api value

        Returns: dualmcu value for this role
        """
        val = 0
        if self.base == self.BaseRole.SINK:
            val += 1
        elif self.base == self.BaseRole.ROUTER:
            val += 2
        elif self.base == self.BaseRole.NON_ROUTER:
            val += 3

        if self.RoleFlags.CSMA_CA in self.flags:
            val += 0x10

        if self.RoleFlags.AUTOROLE in self.flags:
            val += 0x80

        return val

    def __str__(self):
        flag_str = ""
        for flag in self.flags:
            flag_str += str(flag.name)
            flag_str += " "

        return "%s, %s" % (self.base.name, flag_str)


class SinkConfigurator(BusClient):
    """Simple class to configure a node

    Use the Dbus api to set or read nodes configuration
    """

    def configure(
        self,
        sink_name,
        node_address=None,
        node_role=None,
        network_address=None,
        network_channel=None,
        start=None,
        authentication_key=None,
        cipher_key=None,
    ):
        sink = self.sink_manager.get_sink(sink_name)
        if sink is None:
            print("Cannot retrieve sink object with name %s" % sink_name)
            return

        # Do the actual configuration
        config = {}
        if node_address is not None:
            config["node_address"] = node_address
        if node_role is not None:
            config["node_role"] = node_role.to_dualmcu_value()
        if network_address is not None:
            config["network_address"] = network_address
        if network_channel is not None:
            config["network_channel"] = network_channel
        if start is not None:
            config["started"] = start
        if cipher_key is not None:
            config["cipher_key"] = cipher_key
        if authentication_key is not None:
            config["authentication_key"] = authentication_key

        ret = sink.write_config(config)
        print("Configuration done with result = {}".format(ret))

    def list_sinks(self):
        sinks = self.sink_manager.get_sinks()

        print("List of sinks:")
        for sink in sinks:
            print("============== [%s] ===============" % sink.sink_id)
            config, _ = sink.read_config()
            for key in config.keys():
                if key == "node_role":
                    print("[%s]: %s" % (key, NodeRole.from_dualmcu_value(config[key])))
                elif key == "app_config_data":
                    print("[%s]: %s" % (key, binascii.hexlify(config[key])))
                else:
                    print("[%s]: %s" % (key, config[key]))
            print("===================================")


def key_type(param_str):
    """Ensure key type conversion

    Args:
        param_str: the key value as a string given by user

    Returns: The key as a bytearray object
    """
    key_str = param_str.replace(",", " ")
    key = bytearray.fromhex(key_str)

    if len(key) != 16:
        raise argparse.ArgumentTypeError("Key is not 128 bits long")

    return key


def node_role_type(param_str):
    """Ensure node role conversion

    Args:
        param_str: the parameter value as a string given by user

    Returns: The role as a NodeRole object
    """
    try:
        return NodeRole.from_string(param_str)
    except ValueError as e:
        raise argparse.ArgumentTypeError(str(e))


def bool_type(param_str):
    """Ensures string to bool conversion

    Args:
        param_str: the parameter value as a string given by user

    Returns: The value as a boolean
    """
    if param_str.lower() in ("yes", "true", "t", "y", "1"):
        return True
    elif param_str.lower() in ("no", "false", "f", "n", "0", ""):
        return False
    else:
        raise argparse.ArgumentTypeError("Boolean value expected.")


def int_type(param_str):
    """ Ensures string to int conversion to accept 0x format

    Args:
        param_str: the parameter value as a string given by user

    Returns: The value as a boolean
    """
    try:
        value = int(param_str, 0)
    except ValueError:
        raise argparse.ArgumentTypeError("Integer value expected.")
    return value


def get_default_value_from_env(env_var_name):
    value = os.environ.get(env_var_name, None)
    if value is not None and value == "":
        return None
    else:
        return value


def main():
    """Main service to configure a node locally on the gateway

    This tool allows to interact locally on the gateway to configure a node
    with dbus api
    """

    parser = argparse.ArgumentParser()
    parser.add_argument("cmd", choices=["list", "set"])

    parser.add_argument(
        "-s",
        "--sink_name",
        type=str,
        default=get_default_value_from_env("WM_CN_SINK_ID"),
        help="Sink name as configured in sinkService. Ex: -s sink0",
    )

    parser.add_argument(
        "-n",
        "--node_address",
        type=int_type,
        default=get_default_value_from_env("WM_CN_NODE_ADDRESS"),
        help="Node address as an int. Ex: -n 0xABC or -n 123",
    )

    parser.add_argument(
        "-r",
        "--node_role",
        type=node_role_type,
        default=get_default_value_from_env("WM_CN_NODE_ROLE"),
        help="Node role as a string being any combination of a base role "
        "sink/router/non-router and list of flags from [csma-ca, autorole]."
        'Ex: -r "sink csma-ca" or -r "router csma-ca autorole"',
    )

    parser.add_argument(
        "-N",
        "--network_address",
        type=int_type,
        default=get_default_value_from_env("WM_CN_NETWORK_ADDRESS"),
        help="Network address as an int. Ex: -N 0xA1B2C3 or -N 123456",
    )

    parser.add_argument(
        "-c",
        "--network_channel",
        type=int_type,
        default=get_default_value_from_env("WM_CN_NETWORK_CHANNEL"),
        help="Network channel as an int. Ex: -c 5",
    )

    parser.add_argument(
        "-ak",
        "--authentication_key",
        type=key_type,
        default=get_default_value_from_env("WM_CN_AUTHENTICATION_KEY"),
        help="Network wide 128 bytes authentication key. "
        "Ex: -ak 112233445566778899AABBCCDDEEFF11 "
        "or -ak 11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF,11",
    )

    parser.add_argument(
        "-ck",
        "--cipher_key",
        type=key_type,
        default=get_default_value_from_env("WM_CN_CIPHER_KEY"),
        help="Network wide cipher key. "
        "Ex: -ck 112233445566778899AABBCCDDEEFF11 "
        "or -ck 11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF,11",
    )

    parser.add_argument(
        "-S",
        "--start",
        type=bool_type,
        default=get_default_value_from_env("WM_CN_START_SINK"),
        help="Start the sink after configuration",
    )

    args = parser.parse_args()

    sink_configurator = SinkConfigurator()

    if args.cmd == "list":
        sink_configurator.list_sinks()

    if args.cmd == "set":
        sink_configurator.configure(
            node_address=args.node_address,
            node_role=args.node_role,
            network_address=args.network_address,
            network_channel=args.network_channel,
            sink_name=args.sink_name,
            start=args.start,
            authentication_key=args.authentication_key,
            cipher_key=args.cipher_key,
        )


if __name__ == "__main__":
    main()



"""
    Wirepas Transport Service
    =========================

    Installation script

    .. Copyright:
        Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
        See file LICENSE for full license details.

"""

import os
import re

from setuptools import setup, find_packages, Extension
import pkgconfig

here = os.path.abspath(os.path.dirname(__file__))
readme_file = "README.md"
license_file = "LICENSE"

with open(readme_file) as f:
    long_description = f.read()


def get_list_files(root, flist=None):
    if flist is None:
        flist = list()

    for path, subdirs, files in os.walk(root):
        for name in files:
            flist.append(os.path.join(path, name))
    return flist


def get_absolute_path(*args):
    """ Transform relative pathnames into absolute pathnames """
    return os.path.join(here, *args)


def get_requirements(*args):
    """ Get requirements requirements.txt """
    requirements = set()
    with open(get_absolute_path(*args)) as handle:
        for line in handle:
            # Strip comments.
            line = re.sub(r"^#.*|\s#.*", "", line)
            # Ignore empty lines
            if line and not line.isspace():
                requirements.add(re.sub(r"\s+", "", line))
    return sorted(requirements)


def get_systemd_lib(key):
    try:
        return pkgconfig.parse("libsystemd")[key]
    except KeyError:
        return None
    except Exception:
        # Package config is not installed or system pkg-config
        # Default to systemd
        if key == "libraries":
            return ["systemd"]
        else:
            return None


about = {}
with open(get_absolute_path("./wirepas_gateway/__about__.py")) as f:
    exec(f.read(), about)

setup(
    name=about["__pkg_name__"],
    version=about["__version__"],
    description=about["__description__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=about["__author__"],
    author_email=about["__author_email__"],
    url=about["__url__"],
    license=about["__license__"],
    classifiers=about["__classifiers__"],
    keywords=about["__keywords__"],
    packages=find_packages(exclude=["contrib", "docs", "tests", "examples"]),
    install_requires=get_requirements("requirements.txt"),
    python_requires=">=3.7",
    ext_modules=[
        Extension(
            "dbusCExtension",
            sources=["wirepas_gateway/dbus/c-extension/dbus_c.c"],
            libraries=get_systemd_lib("libraries"),
            include_dirs=get_systemd_lib("include_dirs"),
            library_dirs=get_systemd_lib("library_dirs"),
        )
    ],
    include_package_data=True,
    data_files=[
        (
            "./wirepas_gateway-extras/package",
            [readme_file, license_file, "requirements.txt", "setup.py"],
        )
    ],
    entry_points={
        "console_scripts": [
            "wm-gw=wirepas_gateway.transport_service:main",
            "wm-dbus-print=wirepas_gateway.dbus_print_client:main",
            "wm-node-conf=wirepas_gateway.configure_node:main",
        ]
    },
)



"""
    .. Copyright:
        Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
        See file LICENSE for full license details.
"""

__author__ = "Wirepas Ltd"
__author_email__ = "opensource@wirepas.com"
__classifiers__ = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: Apache Software License",
    "Topic :: Software Development :: Libraries",
    "Programming Language :: Python :: 3",
]
__copyright__ = "2019 Wirepas Ltd"
__description__ = "Wirepas gateway transport service that connects the local dbus to a remote MQTT broker."
__license__ = "Apache-2"
__pkg_name__ = "wirepas_gateway"
__title__ = "Wirepas Gateway Transport Service"
__url__ = "https://github.com/wirepas/gateway"
__version__ = "1.4.5rc1"
__keywords__ = "wirepas connectivity iot mesh"



# Copyright 2019 Wirepas Ltd licensed under Apache License, Version 2.0
#
# See file LICENSE for full license details.

import logging
import os
import sys
from datetime import datetime

from wirepas_gateway.dbus.dbus_client import BusClient


class PrintClient(BusClient):
    """
    Simple class example that print all received message from dbus
    """

    def on_data_received(
        self,
        sink_id,
        timestamp,
        src,
        dst,
        src_ep,
        dst_ep,
        travel_time,
        qos,
        hop_count,
        data,
    ):
        """ logs incoming data from the WM network """
        logging.info(
            "Sink %s FROM %d TO %d on EP %d Data Size is %d",
            sink_id,
            src,
            dst,
            dst_ep,
            len(data),
        )

    def on_sink_connected(self, name):
        sink = self.sink_manager.get_sink(name)

        if sink is not None:
            # Read Stack status of sink on connection
            config, _ = sink.read_config()
            logging.info("Sink connected with config: %s", config)


def main():

    # Set default debug level
    debug_level = "info"
    try:
        debug_level = os.environ["DEBUG_LEVEL"]
        print(
            "Deprecated environment variable DEBUG_LEVEL "
            "(it will be dropped from version 2.x onwards)"
            " please use WM_DEBUG_LEVEL instead."
        )
    except KeyError:
        pass

    try:
        debug_level = os.environ["WM_DEBUG_LEVEL"]
    except KeyError:
        pass

    debug_level = "{0}".format(debug_level.upper())
    logging.basicConfig(
        format='%(asctime)s | [%(levelname)s] %(message)s',
        level=debug_level,
        stream=sys.stdout
    )

    PrintClient().run()


if __name__ == "__main__":

    main()
